// for file I/O
use std::fs::File;
use std::io::{Write, BufReader, BufRead, Error};
// for timestamping and profiling
use chrono;
use std::time::Instant;
// for mustache
use mustache;
use std::str;
use std::collections::HashMap;

const PUBLISH_PATH: &str = "./pub/";
const TEMPLATE_PATH: &str = "./src/templates/";

fn render_html(file: &str, template_file: &str) {
    let now = Instant::now(); // start profiling
    let output_path = &format!("{}{}", PUBLISH_PATH, file);
    let template_path = &format!("{}{}", TEMPLATE_PATH, template_file);

    // templating
    let template = mustache::compile_path(template_path).expect("Failed to compile");
    
    let mut data = HashMap::new();
    data.insert("h1", "Hello World");

    let mut bytes = vec![];

    template
        .render(&mut bytes, &data)
        .expect("Failed to render");

    let html_string = str::from_utf8(&bytes).unwrap();

    let output = format!(
        "{}\n<!--\n  {}\n  {}\n  generated in {:.2?}\n-->",
        html_string,
        template_path,
        chrono::offset::Local::now(),
        now.elapsed(),
    );

    // write output to file

    let result = write_file(output_path, output);
    match result {
        Ok(file) => file,
        Err(e) => panic!("Problem creating {:?}", e)
    }

    /*
    let input = File::open(output_path)?;
    let buffered = BufReader::new(input);

    for line in buffered.lines() {
        println!("{}", line?);
    }
    */

}

fn write_file(file_path: &str, contents: String) -> Result<(), Error> {
    let mut output_file = File::create(file_path)?;
    write!(output_file, "{}", contents)?;
    Ok(())
}

fn main() {

    render_html("index.html", "index.mustache");

}
